@@ -13,9 +13,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-embed-event.c 6589 2006-09-13 11:49:01Z chpe $
  */
 
 #include "config.h"
@@ -23,8 +22,8 @@
 #include "ephy-embed-event.h"
 #include "ephy-embed-type-builtins.h"
 
-#include <glib/ghash.h>
-#include <gtk/gtktypeutils.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 static void ephy_embed_event_base_init (gpointer g_class);
 
