@@ -29,26 +29,25 @@
 #include "eel-gconf-extensions.h"
 #include "ephy-command-manager.h"
 #include "ephy-debug.h"
-#include "ephy-file-chooser.h"
-#include "ephy-file-helpers.h"
-#include "ephy-history.h"
+#include "ephy-embed.h"
+#include "ephy-embed-event.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
 #include "ephy-embed-persist.h"
+#include "ephy-embed-prefs.h"
+#include "ephy-embed-utils.h"
+#include "ephy-file-chooser.h"
+#include "ephy-file-helpers.h"
+#include "ephy-history.h"
+#include "ephy-prefs.h"
 #include "ephy-stock-icons.h"
 #include "ephy-string.h"
-#include "ephy-embed-event.h"
-#include "ephy-embed-utils.h"
-#include "ephy-prefs.h"
 #include "ephy-web-view.h"
 
-#include <webkit/webkit.h>
 #include <errno.h>
-#include <string.h>
 #include <glib/gi18n.h>
-
-#include "ephy-embed-prefs.h"
-#include "ephy-embed.h"
+#include <string.h>
+#include <webkit/webkit.h>
 
 static void     ephy_embed_class_init  (EphyEmbedClass *klass);
 static void     ephy_embed_init        (EphyEmbed *gs);
@@ -599,7 +598,7 @@ confirm_action_from_mime (WebKitWebView *web_view,
 
   g_free (mime_description);
 
-  button = gtk_button_new_with_label (_("_Save As..."));
+  button = gtk_button_new_with_mnemonic (_("_Save As..."));
   image = gtk_image_new_from_stock (GTK_STOCK_SAVE_AS, GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
   /* don't show the image! see bug #307818 */
