@@ -2,6 +2,8 @@
  *  Copyright © 2002 Jorn Baayen
  *  Copyright © 2003 Marco Pesenti Gritti
  *  Copyright © 2003, 2004 Christian Persch
+ *  Copyright © 2009 Igalia S.L.
+ *  Copyright © 2009 Holger Hans Peter Freyther
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,16 +19,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #include "config.h"
 
 #include "pdm-dialog.h"
 #include "ephy-shell.h"
-#include "ephy-cookie-manager.h"
 #include "ephy-file-helpers.h"
-#include "ephy-password-manager.h"
 #include "ephy-gui.h"
 #include "ephy-state.h"
 #include "ephy-string.h"
@@ -36,14 +35,20 @@
 #include "ephy-embed-single.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-history.h"
+#include "ephy-password-info.h"
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <libsoup/soup.h>
+#include <webkit/webkit.h>
+#include <gnome-keyring.h>
+#include <gnome-keyring-memory.h>
 
 #include <string.h>
+#include <time.h>
 
 typedef struct PdmActionInfo PdmActionInfo;
-	
+
 struct PdmActionInfo
 {
 	/* Methods */
@@ -68,6 +73,13 @@ struct PdmActionInfo
 	gboolean delete_row_on_remove;
 };
 
+typedef struct PdmCallBackData PdmCallBackData;
+struct PdmCallBackData
+{
+    guint key;
+    GtkListStore *store;
+};
+
 #define EPHY_PDM_DIALOG_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_PDM_DIALOG, PdmDialogPrivate))
 
 struct PdmDialogPrivate
@@ -97,7 +109,7 @@ enum
 	COL_PASSWORDS_HOST,
 	COL_PASSWORDS_USER,
 	COL_PASSWORDS_PASSWORD,
-	COL_PASSWORDS_DATA
+	COL_PASSWORDS_DATA,
 };
 
 enum
@@ -123,12 +135,12 @@ EphyDialogProperty properties [] =
 	{ "pdm_dialog",			NULL, PT_NORMAL, 0 },
 	{ "pdm_notebook",		NULL, PT_NORMAL, 0 },
 
-	{ "cookies_treeview",	   	NULL, PT_NORMAL, 0 },
-	{ "cookies_remove_button",     	NULL, PT_NORMAL, 0 },
-	{ "cookies_properties_button", 	NULL, PT_NORMAL, 0 },
-	{ "passwords_treeview",	       	NULL, PT_NORMAL, 0 },
-	{ "passwords_remove_button",   	NULL, PT_NORMAL, 0 },
-	{ "passwords_show_button",     	NULL, PT_NORMAL, 0 },
+	{ "cookies_treeview",		NULL, PT_NORMAL, 0 },
+	{ "cookies_remove_button",	NULL, PT_NORMAL, 0 },
+	{ "cookies_properties_button",	NULL, PT_NORMAL, 0 },
+	{ "passwords_treeview",		NULL, PT_NORMAL, 0 },
+	{ "passwords_remove_button",	NULL, PT_NORMAL, 0 },
+	{ "passwords_show_button",	NULL, PT_NORMAL, 0 },
 
 	{ NULL }
 };
@@ -136,10 +148,7 @@ EphyDialogProperty properties [] =
 static void pdm_dialog_class_init	(PdmDialogClass *klass);
 static void pdm_dialog_init		(PdmDialog *dialog);
 static void pdm_dialog_finalize		(GObject *object);
-
-static void passwords_changed_cb 		(EphyPasswordManager *manager,
-						 PdmDialog *dialog);
-
+static void pdm_dialog_password_remove (PdmActionInfo *info, gpointer data);
 
 G_DEFINE_TYPE (PdmDialog, pdm_dialog, EPHY_TYPE_DIALOG)
 
@@ -186,12 +195,57 @@ typedef struct
 	guint num_checked;
 } PdmClearAllDialogButtons;
 
+static SoupCookieJar*
+get_cookie_jar ()
+{
+	SoupSession* session;
+
+	session = webkit_get_default_session ();
+	return (SoupCookieJar*)soup_session_get_feature (session, SOUP_TYPE_COOKIE_JAR);
+}
+
 static void
 clear_all_dialog_release_cb (PdmClearAllDialogButtons *data)
 {
 	g_slice_free (PdmClearAllDialogButtons, data);
 }
 
+static void
+clear_all_cookies (SoupCookieJar *jar)
+{
+	GSList *l, *p;
+
+	l = soup_cookie_jar_all_cookies (jar);
+	for (p = l; p; p = p->next)
+		soup_cookie_jar_delete_cookie (jar, (SoupCookie*)p->data);
+
+	soup_cookies_free (l);
+}
+
+static void
+pdm_dialog_password_remove_cb (GnomeKeyringResult result,
+			       gpointer data)
+{
+	GtkTreeRowReference *rowref = (GtkTreeRowReference *)data;
+
+	if (result == GNOME_KEYRING_RESULT_OK) {
+		GtkTreeIter iter;
+		GtkTreePath *path;
+		GtkTreeModel *model;
+
+		if (!gtk_tree_row_reference_valid (rowref))
+			return;
+
+		path = gtk_tree_row_reference_get_path (rowref);
+		model = gtk_tree_row_reference_get_model (rowref);
+
+		if (path != NULL && gtk_tree_model_get_iter (model, &iter, path)) {
+			gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+			gtk_tree_path_free (path);
+		}
+	}
+}
+
 static void
 clear_all_dialog_response_cb (GtkDialog *dialog,
 			      int response,
@@ -208,7 +262,7 @@ clear_all_dialog_response_cb (GtkDialog *dialog,
 	if (response == GTK_RESPONSE_OK)
 	{
 		if (gtk_toggle_button_get_active 
-		    	(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_history)))
+			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_history)))
 		{
 			EphyEmbedShell *shell;
 			EphyHistory *history;
@@ -218,51 +272,48 @@ clear_all_dialog_response_cb (GtkDialog *dialog,
 			ephy_history_clear (history);
 		}
 		if (gtk_toggle_button_get_active
-		    	(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_cookies)))
+			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_cookies)))
 		{
-			EphyCookieManager *manager;
+			SoupCookieJar *jar;
 
-			manager = EPHY_COOKIE_MANAGER (ephy_embed_shell_get_embed_single
-						       (EPHY_EMBED_SHELL (ephy_shell)));
-
-			ephy_cookie_manager_clear (manager);
+			jar = get_cookie_jar ();
+			clear_all_cookies (jar);
 		}
 		if (gtk_toggle_button_get_active
-		    	(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_passwords)))
+			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_passwords)))
 		{
-			EphyPasswordManager *manager;
+			GtkTreeIter iter;
+			PdmDialog *pdialog = EPHY_PDM_DIALOG (checkbuttons->dialog);
+			PdmActionInfo *pinfo = pdialog->priv->passwords;
 
-			manager = EPHY_PASSWORD_MANAGER (ephy_embed_shell_get_embed_single
-							 (EPHY_EMBED_SHELL (ephy_shell)));
+			gboolean valid = gtk_tree_model_get_iter_first (pinfo->model, &iter);
 
-			/* we don't remove the password from the liststore in the callback
-			 * like we do for cookies, since the callback doesn't carry that
-			 * information, and we'd have to reload the whole list, losing the
-			 * selection in the process.
-			 */
+			while (valid) {
+				GtkTreePath *path;
+				EphyPasswordInfo *info;
+				GtkTreeRowReference *row;
 
-			if (EPHY_IS_PDM_DIALOG (checkbuttons->dialog))
-			{
-				g_signal_handlers_block_by_func
-					(manager, G_CALLBACK (passwords_changed_cb), 
-					 EPHY_PDM_DIALOG (checkbuttons->dialog));
-			}
+				path = gtk_tree_model_get_path (pinfo->model, &iter);
+				row = gtk_tree_row_reference_new (pinfo->model, path);
 
-			/* Flush the list */
-			ephy_password_manager_remove_all_passwords (manager);
+				gtk_tree_model_get (pinfo->model, &iter,
+						    COL_PASSWORDS_DATA, &info,
+						    -1);
 
-			if (EPHY_IS_PDM_DIALOG (checkbuttons->dialog))
-			{
-				g_signal_handlers_unblock_by_func
-					(manager, G_CALLBACK (passwords_changed_cb),
-					 EPHY_PDM_DIALOG (checkbuttons->dialog));
+				gnome_keyring_item_delete (GNOME_KEYRING_DEFAULT,
+							   info->keyring_id,
+							   (GnomeKeyringOperationDoneCallback) pdm_dialog_password_remove_cb,
+							   row,
+							   (GDestroyNotify) gtk_tree_row_reference_free);
 
-				/* And now refresh explicitly */
-				passwords_changed_cb (manager, EPHY_PDM_DIALOG (checkbuttons->dialog));
+				valid = gtk_tree_model_iter_next (pinfo->model, &iter);
+
+				g_slice_free (EphyPasswordInfo, info);
+				gtk_tree_path_free (path);
 			}
 		}
 		if (gtk_toggle_button_get_active
-		    	(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_cache)))
+			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_cache)))
 		{
 			EphyEmbedShell *shell;
 			EphyEmbedSingle *single;
@@ -287,7 +338,7 @@ clear_all_dialog_checkbutton_toggled_cb (GtkToggleButton *toggle,
 {
 	GtkWidget *dialog;
 	dialog = gtk_widget_get_toplevel (GTK_WIDGET (toggle));
-	
+
 	if (gtk_toggle_button_get_active (toggle) == TRUE)
 	{
 		data->num_checked++;
@@ -299,6 +350,7 @@ clear_all_dialog_checkbutton_toggled_cb (GtkToggleButton *toggle,
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK,
 					   data->num_checked != 0);
 }
+
 void
 pdm_dialog_show_clear_all_dialog (EphyDialog *edialog,
 				  GtkWidget *parent,
@@ -307,6 +359,7 @@ pdm_dialog_show_clear_all_dialog (EphyDialog *edialog,
 	GtkWidget *dialog, *vbox;
 	GtkWidget *check, *label;
 	PdmClearAllDialogButtons *checkbuttons;
+	GtkWidget *button, *icon;
 
 	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (parent),
 						     GTK_DIALOG_DESTROY_WITH_PARENT |
@@ -322,14 +375,22 @@ pdm_dialog_show_clear_all_dialog (EphyDialog *edialog,
 						    "check the types of information that you "
 						    "want to remove:"));
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Clear All Personal Data"));
+
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 				GTK_STOCK_HELP,
 				GTK_RESPONSE_HELP,
 				GTK_STOCK_CANCEL,
 				GTK_RESPONSE_CANCEL,
-				GTK_STOCK_CLEAR,
-				GTK_RESPONSE_OK,
 				NULL);
+
+	/* Clear button */
+	button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+					_("Cl_ear"),
+					GTK_RESPONSE_OK);
+	icon = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_BUTTON);
+	gtk_button_set_image (GTK_BUTTON (button), icon);
+	gtk_widget_show (button);
+
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 					 GTK_RESPONSE_CANCEL);
 	gtk_label_set_selectable (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label),
@@ -368,7 +429,7 @@ pdm_dialog_show_clear_all_dialog (EphyDialog *edialog,
 	}
 
 	/* History */
-	check = gtk_check_button_new_with_mnemonic (_("_History"));
+	check = gtk_check_button_new_with_mnemonic (_("Hi_story"));
 	checkbuttons->checkbutton_history = check;
 	gtk_box_pack_start (GTK_BOX (vbox), check,
 			    FALSE, FALSE, 0);
@@ -378,7 +439,7 @@ pdm_dialog_show_clear_all_dialog (EphyDialog *edialog,
 	{
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
 	}
-	
+
 	/* Cache */
 	check = gtk_check_button_new_with_mnemonic (_("_Temporary files"));
 	checkbuttons->checkbutton_cache = check;
@@ -397,7 +458,7 @@ pdm_dialog_show_clear_all_dialog (EphyDialog *edialog,
 	label = gtk_label_new (NULL);
 	gtk_label_set_markup (GTK_LABEL (label),
 			      _("<small><i><b>Note:</b> You cannot undo this action. "
-			        "The data you are choosing to clear "
+				"The data you are choosing to clear "
 				"will be deleted forever.</i></small>"));
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    label, FALSE, FALSE, 0);
@@ -460,14 +521,14 @@ pdm_cmd_delete_selection (PdmActionInfo *action)
 	}
 
 	/* Intelligent selection logic, no actual selection yet */
-	
-	path = gtk_tree_row_reference_get_path 
+
+	path = gtk_tree_row_reference_get_path
 		((GtkTreeRowReference *) g_list_first (rlist)->data);
-	
+
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_path_free (path);
 	iter2 = iter;
-	
+
 	if (gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter))
 	{
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
@@ -482,16 +543,13 @@ pdm_cmd_delete_selection (PdmActionInfo *action)
 		}
 	}
 	gtk_tree_path_free (path);
-	
+
 	/* Removal */
-	
 	for (r = rlist; r != NULL; r = r->next)
 	{
 		GValue val = { 0, };
 
-		path = gtk_tree_row_reference_get_path
-			((GtkTreeRowReference *)r->data);
-
+		path = gtk_tree_row_reference_get_path ((GtkTreeRowReference *)r->data);
 		gtk_tree_model_get_iter (model, &iter, path);
 		gtk_tree_model_get_value (model, &iter, action->data_col, &val);
 		action->remove (action, g_value_get_boxed (&val));
@@ -503,6 +561,7 @@ pdm_cmd_delete_selection (PdmActionInfo *action)
 			gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
 		}
 
+		gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
 		gtk_tree_row_reference_free ((GtkTreeRowReference *)r->data);
 		gtk_tree_path_free (path);
 	}
@@ -512,7 +571,6 @@ pdm_cmd_delete_selection (PdmActionInfo *action)
 	g_list_free (rlist);
 
 	/* Selection */
-	
 	if (row_ref != NULL)
 	{
 		path = gtk_tree_row_reference_get_path (row_ref);
@@ -579,7 +637,7 @@ setup_action (PdmActionInfo *action)
 
 static void
 show_cookies_properties (PdmDialog *dialog,
-			 EphyCookie *info)
+			 SoupCookie *info)
 {
 	GtkWidget *gdialog;
 	GtkWidget *table;
@@ -596,7 +654,7 @@ show_cookies_properties (PdmDialog *dialog,
 		  GTK_WINDOW (parent),
 		  GTK_DIALOG_DESTROY_WITH_PARENT,
 		  GTK_STOCK_CLOSE, 0, NULL);
-	ephy_state_add_window (GTK_WIDGET (gdialog), "cookie_properties", 
+	ephy_state_add_window (GTK_WIDGET (gdialog), "cookie_properties",
 			       -1, -1, FALSE,
 			       EPHY_STATE_WINDOW_SAVE_SIZE | EPHY_STATE_WINDOW_SAVE_POSITION);
 	gtk_dialog_set_has_separator (GTK_DIALOG(gdialog), FALSE);
@@ -650,7 +708,7 @@ show_cookies_properties (PdmDialog *dialog,
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
 			  GTK_FILL, GTK_FILL, 0, 0);
 
-	label = gtk_label_new (info->is_secure ? _("Encrypted connections only") : _("Any type of connection") );
+	label = gtk_label_new (info->secure ? _("Encrypted connections only") : _("Any type of connection") );
 	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
 	gtk_widget_show (label);
@@ -665,14 +723,16 @@ show_cookies_properties (PdmDialog *dialog,
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
 			  GTK_FILL, GTK_FILL, 0, 0);
 
-	if (info->is_session)
+	if (info->expires == NULL)
 	{
+		/* Session cookie */
 		str = g_strdup (_("End of current session"));
 	}
 	else
 	{
 		struct tm t;
-		str = eel_strdup_strftime ("%c", localtime_r (&info->expires, &t));
+		time_t out = soup_date_to_time_t(info->expires);
+		str = eel_strdup_strftime ("%c", localtime_r (&out, &t));
 	}
 	label = gtk_label_new (str);
 	g_free (str);
@@ -702,19 +762,18 @@ cookies_properties_clicked_cb (GtkWidget *button,
 	GValue val = {0, };
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	EphyCookie *cookie;
+	SoupCookie *cookie;
 	GList *l;
 	GtkTreeSelection *selection;
 
 	selection = gtk_tree_view_get_selection (dialog->priv->cookies->treeview);
-	l = gtk_tree_selection_get_selected_rows
-		(selection, &model);
+	l = gtk_tree_selection_get_selected_rows (selection, &model);
 
 	path = (GtkTreePath *)l->data;
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_model_get_value
 		(model, &iter, COL_COOKIES_DATA, &val);
-	cookie = (EphyCookie *) g_value_get_boxed (&val);
+	cookie = (SoupCookie *) g_value_get_boxed (&val);
 
 	show_cookies_properties (dialog, cookie);
 
@@ -783,7 +842,7 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 					G_TYPE_STRING,
 					G_TYPE_STRING,
 					G_TYPE_STRING,
-					EPHY_TYPE_COOKIE);
+					SOUP_TYPE_COOKIE);
 	gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(liststore));
 	gtk_tree_view_set_headers_visible (treeview, TRUE);
 	selection = gtk_tree_view_get_selection (treeview);
@@ -792,7 +851,6 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 	info->model = GTK_TREE_MODEL (liststore);
 	g_object_unref (liststore);
 
-        cookies_treeview_selection_changed_cb (selection, dialog);
 	g_signal_connect (selection, "changed",
 			  G_CALLBACK(cookies_treeview_selection_changed_cb),
 			  dialog);
@@ -835,20 +893,9 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 	setup_action (info);
 }
 
-static gboolean
-compare_cookies (const EphyCookie *cookie1,
-		 const EphyCookie *cookie2)
-{
-	g_return_val_if_fail (cookie1 != NULL || cookie2 != NULL, FALSE);
-
-	return (strcmp (cookie1->domain, cookie2->domain) == 0
-		&& strcmp (cookie1->path, cookie2->path) == 0
-		&& strcmp (cookie1->name, cookie2->name) == 0);
-}
-
 static gboolean
 cookie_to_iter (GtkTreeModel *model,
-		const EphyCookie *cookie,
+		const SoupCookie *cookie,
 		GtkTreeIter *iter)
 {
 	gboolean valid;
@@ -858,15 +905,15 @@ cookie_to_iter (GtkTreeModel *model,
 
 	while (valid)
 	{
-		EphyCookie *data;
+		SoupCookie *data;
 
 		gtk_tree_model_get (model, iter,
 				    COL_COOKIES_DATA, &data,
 				    -1);
 
-		found = compare_cookies (cookie, data);
+		found = soup_cookie_equal ((SoupCookie*)cookie, data);
 
-		ephy_cookie_free (data);
+		soup_cookie_free (data);
 
 		if (found) break;
 
@@ -877,69 +924,43 @@ cookie_to_iter (GtkTreeModel *model,
 }
 
 static void
-cookie_added_cb (EphyCookieManager *manager,
-		 const EphyCookie *cookie,
-		 PdmDialog *dialog)
-{
-	PdmActionInfo *info = dialog->priv->cookies;
-	
-	LOG ("cookie_added_cb");
-
-	info->add (info, (gpointer) ephy_cookie_copy (cookie));
-}
-
-static void
-cookie_changed_cb (EphyCookieManager *manager,
-		   const EphyCookie *cookie,
+cookie_changed_cb (SoupCookieJar *jar,
+		   const SoupCookie *old_cookie,
+		   const SoupCookie *new_cookie,
 		   PdmDialog *dialog)
 {
-	PdmActionInfo *info = dialog->priv->cookies;
+	PdmActionInfo *info;
 	GtkTreeIter iter;
 
+	g_return_if_fail (EPHY_IS_PDM_DIALOG (dialog));
+	info = dialog->priv->cookies;
+
 	LOG ("cookie_changed_cb");
 
-	if (cookie_to_iter (info->model, cookie, &iter))
+	g_return_if_fail (info);
+
+	if (old_cookie)
 	{
-		gtk_list_store_remove (GTK_LIST_STORE (info->model), &iter);		
-		info->add (info, (gpointer) ephy_cookie_copy (cookie));
+		/* Cookie changed or deleted, let's get rid of the old one
+		   in any case */
+		if (cookie_to_iter (info->model, old_cookie, &iter))
+		{
+			gtk_list_store_remove (GTK_LIST_STORE (info->model), &iter);
+		}
+		else
+		{
+			g_warning ("Unable to find changed cookie in list!\n");
+		}
 	}
-	else
+
+	if (new_cookie)
 	{
-		g_warning ("Unable to find changed cookie in list!\n");
+		/* Cookie changed or added, let's add the new cookie in
+		   any case */
+		info->add (info, (gpointer) soup_cookie_copy ((SoupCookie*)new_cookie));
 	}
 }
 
-static void
-cookie_deleted_cb (EphyCookieManager *manager,
-		   const EphyCookie *cookie,
-		   PdmDialog *dialog)
-{
-	PdmActionInfo *info = dialog->priv->cookies;
-	GtkTreeIter iter;
-
-	LOG ("cookie_deleted_cb");
-
-	if (cookie_to_iter (info->model, cookie, &iter))
-	{
-		gtk_list_store_remove (GTK_LIST_STORE (info->model), &iter);		
-	}
-	else
-	{
-		g_warning ("Unable to find deleted cookie in list!\n");
-	}
-}
-
-static void
-cookies_cleared_cb (EphyCookieManager *manager,
-		    PdmDialog *dialog)
-{
-	PdmActionInfo *info = dialog->priv->cookies;
-
-	LOG ("cookies_cleared_cb");
-
-	gtk_list_store_clear (GTK_LIST_STORE (info->model));
-}
-
 static gboolean
 cookie_host_to_iter (GtkTreeModel *model,
 		     const char *key1,
@@ -969,7 +990,7 @@ cookie_host_to_iter (GtkTreeModel *model,
 		/* Start on the \0 */
 		p = key1 + len;
 		q = key2 + strlen (key2);
-	
+
 		do
 		{
 			if (*p == '.') ++n;
@@ -1031,15 +1052,13 @@ compare_cookie_host_keys (GtkTreeModel *model,
 static void
 pdm_dialog_fill_cookies_list (PdmActionInfo *info)
 {
-	EphyCookieManager *manager;
-	GList *list, *l;
+	SoupCookieJar *jar;
+	GSList *list, *l;
 
 	g_assert (info->filled == FALSE);
 
-	manager = EPHY_COOKIE_MANAGER (ephy_embed_shell_get_embed_single
-			(EPHY_EMBED_SHELL (ephy_shell)));
-
-	list = ephy_cookie_manager_list_cookies (manager);
+	jar = get_cookie_jar ();
+	list = soup_cookie_jar_all_cookies (jar);
 
 	for (l = list; l != NULL; l = l->next)
 	{
@@ -1047,7 +1066,7 @@ pdm_dialog_fill_cookies_list (PdmActionInfo *info)
 	}
 
 	/* the element data has been consumed, so we need only to free the list */
-	g_list_free (list);
+	g_slist_free (list);
 
 	/* Now turn on sorting */
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (info->model),
@@ -1057,18 +1076,11 @@ pdm_dialog_fill_cookies_list (PdmActionInfo *info)
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (info->model),
 					      COL_COOKIES_HOST_KEY,
 					      GTK_SORT_ASCENDING);
-	
+
 	info->filled = TRUE;
 
-	/* Now connect the callbacks on the EphyCookieManager */
-	g_signal_connect (manager, "cookie-added",
-			 G_CALLBACK (cookie_added_cb), info->dialog);
-	g_signal_connect (manager, "cookie-changed",
-			 G_CALLBACK (cookie_changed_cb), info->dialog);
-	g_signal_connect (manager, "cookie-deleted",
-			 G_CALLBACK (cookie_deleted_cb), info->dialog);
-	g_signal_connect (manager, "cookies-cleared",
-			 G_CALLBACK (cookies_cleared_cb), info->dialog);
+	g_signal_connect (jar, "changed",
+			  G_CALLBACK (cookie_changed_cb), info->dialog);
 
 	info->scroll_to (info);
 }
@@ -1084,7 +1096,7 @@ static void
 pdm_dialog_cookie_add (PdmActionInfo *info,
 		       gpointer data)
 {
-	EphyCookie *cookie = (EphyCookie *) data;
+	SoupCookie *cookie = (SoupCookie *) data;
 	GtkListStore *store;
 	GtkTreeIter iter;
 	int column[4] = { COL_COOKIES_HOST, COL_COOKIES_HOST_KEY, COL_COOKIES_NAME, COL_COOKIES_DATA };
@@ -1101,7 +1113,7 @@ pdm_dialog_cookie_add (PdmActionInfo *info,
 	g_value_init (&value[0], G_TYPE_STRING);
 	g_value_init (&value[1], G_TYPE_STRING);
 	g_value_init (&value[2], G_TYPE_STRING);
-	g_value_init (&value[3], EPHY_TYPE_COOKIE);
+	g_value_init (&value[3], SOUP_TYPE_COOKIE);
 
 	g_value_set_static_string (&value[0], cookie->domain);
 	g_value_take_string (&value[1], ephy_string_collate_key_for_domain (cookie->domain, -1));
@@ -1122,13 +1134,9 @@ static void
 pdm_dialog_cookie_remove (PdmActionInfo *info,
 			  gpointer data)
 {
-	EphyCookie *cookie = (EphyCookie *) data;
-	EphyCookieManager *manager;
+	SoupCookie *cookie = (SoupCookie *) data;
 
-	manager = EPHY_COOKIE_MANAGER (ephy_embed_shell_get_embed_single
-			(EPHY_EMBED_SHELL (ephy_shell)));
-
-	ephy_cookie_manager_remove_cookie (manager, cookie);
+	soup_cookie_jar_delete_cookie (get_cookie_jar(), cookie);
 }
 
 static void
@@ -1155,6 +1163,70 @@ pdm_dialog_cookie_scroll_to (PdmActionInfo *info)
 }
 
 /* "Passwords" tab */
+static void
+passwords_data_func_get_item_cb (GnomeKeyringResult result,
+				 GnomeKeyringItemInfo *info,
+				 gpointer data)
+{
+	GtkTreeRowReference *rowref = (GtkTreeRowReference *)data;
+
+	if (result == GNOME_KEYRING_RESULT_OK) {
+		GtkTreeIter iter;
+		GtkTreePath *path;
+		GtkTreeModel *model;
+
+		if (!gtk_tree_row_reference_valid (rowref))
+			return;
+
+		path = gtk_tree_row_reference_get_path (rowref);
+		model = gtk_tree_row_reference_get_model (rowref);
+
+		if (path != NULL && gtk_tree_model_get_iter (model, &iter, path)) {
+			EphyPasswordInfo *epinfo;
+			GValue val = {0, };
+
+			gtk_tree_model_get_value (model, &iter, COL_PASSWORDS_DATA, &val);
+			epinfo = g_value_get_boxed (&val);
+			epinfo->secret = gnome_keyring_memory_strdup (gnome_keyring_item_info_get_secret (info));
+
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+					    COL_PASSWORDS_DATA, epinfo,
+					    COL_PASSWORDS_PASSWORD, epinfo->secret, -1);
+			g_value_unset (&val);
+		}
+	}
+}
+
+static void
+passwords_data_func (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
+		     GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
+{
+	GValue val = { 0, };
+	EphyPasswordInfo *info;
+
+	if (!gtk_tree_view_column_get_visible (tree_column))
+		return;
+
+	gtk_tree_model_get_value (model, iter, COL_PASSWORDS_DATA, &val);
+	info = g_value_get_boxed (&val);
+
+	if (info->secret == NULL) {
+		GtkTreePath *path;
+		GtkTreeRowReference *rowref;
+
+		path = gtk_tree_model_get_path (model, iter);
+		rowref = gtk_tree_row_reference_new (model, path);
+
+		gnome_keyring_item_get_info_full (GNOME_KEYRING_DEFAULT,
+						  info->keyring_id,
+						  GNOME_KEYRING_ITEM_INFO_SECRET,
+						  (GnomeKeyringOperationGetItemInfoCallback) passwords_data_func_get_item_cb,
+						  rowref,
+						  (GDestroyNotify) gtk_tree_row_reference_free);
+		gtk_tree_path_free (path);
+	}
+	g_value_unset (&val);
+}
 
 static void
 passwords_show_toggled_cb (GtkWidget *button,
@@ -1169,7 +1241,7 @@ passwords_show_toggled_cb (GtkWidget *button,
 	column = gtk_tree_view_get_column (treeview, COL_PASSWORDS_PASSWORD);
 
 	active = gtk_toggle_button_get_active ((GTK_TOGGLE_BUTTON (button)));
-	
+
 	gtk_tree_view_column_set_visible (column, active);
 }
 
@@ -1200,6 +1272,7 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 					G_TYPE_STRING,
 					G_TYPE_STRING,
 					EPHY_TYPE_PASSWORD_INFO);
+
 	gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(liststore));
 	gtk_tree_view_set_headers_visible (treeview, TRUE);
 	selection = gtk_tree_view_get_selection (treeview);
@@ -1222,6 +1295,7 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_sort_column_id (column, COL_PASSWORDS_HOST);
 
+	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (treeview,
 						     COL_PASSWORDS_USER,
 						     _("User Name"),
@@ -1234,6 +1308,7 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_sort_column_id (column, COL_PASSWORDS_USER);
 
+	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (treeview,
 						     COL_PASSWORDS_PASSWORD,
 						     _("User Password"),
@@ -1241,70 +1316,48 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 						     "text", COL_PASSWORDS_PASSWORD,
 						     NULL);
 	column = gtk_tree_view_get_column (treeview, COL_PASSWORDS_PASSWORD);
-	/* Hide this info by default */
+	gtk_tree_view_column_set_cell_data_func (column,
+						 renderer,
+						 passwords_data_func,
+						 info,
+						 NULL);
+	/* Initially shown as hidden colum */
 	gtk_tree_view_column_set_visible (column, FALSE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_sort_column_id (column, COL_PASSWORDS_PASSWORD);
 
 	info->treeview = treeview;
-
 	setup_action (info);
 }
 
 static void
-passwords_changed_cb (EphyPasswordManager *manager,
-		      PdmDialog *dialog)
+pdm_dialog_fill_passwords_list_async_cb (GnomeKeyringResult result,
+					 GList *list,
+					 gpointer data)
 {
-	GtkTreeModel *model = dialog->priv->passwords->model;
+	GList *l;
+	PdmActionInfo *info = (PdmActionInfo *)data;
 
-	LOG ("passwords changed");
+	if (result != GNOME_KEYRING_RESULT_OK)
+		return;
 
-	/* since the callback doesn't carry any information about what
-	 * exactly has changed, we have to rebuild the list from scratch.
-	 */
-	gtk_list_store_clear (GTK_LIST_STORE (model));
+	for (l = list; l != NULL; l = l->next)
+		info->add (info, l->data);
 
-	/* And turn off sorting */
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
-					      GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
+	info->filled = TRUE;
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (info->model),
+					      COL_PASSWORDS_HOST,
 					      GTK_SORT_ASCENDING);
-
-	dialog->priv->passwords->fill (dialog->priv->passwords);
 }
 
 static void
 pdm_dialog_fill_passwords_list (PdmActionInfo *info)
 {
-	EphyPasswordManager *manager;
-	GList *list, *l;
-
-	manager = EPHY_PASSWORD_MANAGER (ephy_embed_shell_get_embed_single
-			(EPHY_EMBED_SHELL (ephy_shell)));
-
-	list = ephy_password_manager_list_passwords (manager);
-
-	for (l = list; l != NULL; l = l->next)
-	{
-		info->add (info, l->data);
-	}
-
-	/* the element data has been consumed, so we need only to free the list */
-	g_list_free (list);
-
-	/* Let's get notified when the list changes */
-	if (info->filled == FALSE)
-	{
-		g_signal_connect (manager, "passwords-changed",
-				  G_CALLBACK (passwords_changed_cb), info->dialog);
-	}
-
-	info->filled = TRUE;
-
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (info->model),
-					      COL_PASSWORDS_HOST,
-					      GTK_SORT_ASCENDING);
+	gnome_keyring_list_item_ids (GNOME_KEYRING_DEFAULT,
+				     pdm_dialog_fill_passwords_list_async_cb,
+				     info,
+				     NULL);
 }
 
 static void
@@ -1312,29 +1365,102 @@ pdm_dialog_passwords_destruct (PdmActionInfo *info)
 {
 }
 
+static void
+pdm_dialog_password_add_item_attrs_cb (GnomeKeyringResult result,
+				       GnomeKeyringAttributeList *attributes,
+				       gpointer data)
+{
+	EphyPasswordInfo *pinfo;
+	PdmCallBackData *cbdata;
+	GnomeKeyringAttribute *attribute;
+	gchar *user, *host, *protocol;
+	GtkTreeIter iter;
+	int i;
+
+	if (result != GNOME_KEYRING_RESULT_OK)
+		return;
+
+	cbdata = (PdmCallBackData *)data;
+
+	user = host = protocol = NULL;
+	attribute = (GnomeKeyringAttribute *) attributes->data;
+	for (i = 0; i < attributes->len; ++i) {
+		if (attribute[i].type == GNOME_KEYRING_ATTRIBUTE_TYPE_STRING) {
+			if (strcmp (attribute[i].name, "server") == 0)
+				host = g_strdup (attribute[i].value.string);
+			else if (strcmp (attribute[i].name, "user") == 0)
+				user = g_strdup (attribute[i].value.string);
+			else if (strcmp (attribute[i].name, "protocol") == 0)
+				protocol = attribute[i].value.string;
+			}
+		}
+	if (!protocol || strncmp("http", protocol, 4) != 0)
+		return;
+
+	pinfo = ephy_password_info_new (cbdata->key);
+	if (!pinfo)
+		return;
+
+	gtk_list_store_append (cbdata->store, &iter);
+	gtk_list_store_set (cbdata->store, &iter,
+			    COL_PASSWORDS_HOST, host,
+			    COL_PASSWORDS_USER, user,
+			    COL_PASSWORDS_PASSWORD, NULL,
+			    COL_PASSWORDS_DATA, pinfo,
+			    -1);
+}
+
+static void
+pdm_dialog_password_add_item_info_cb (GnomeKeyringResult result,
+				      GnomeKeyringItemInfo *info,
+				      gpointer data)
+{
+	if (result != GNOME_KEYRING_RESULT_OK)
+		return;
+
+	if (gnome_keyring_item_info_get_type (info) == GNOME_KEYRING_ITEM_NETWORK_PASSWORD) {
+		PdmCallBackData *cbdata = (PdmCallBackData *)data;
+		gnome_keyring_item_get_attributes (GNOME_KEYRING_DEFAULT,
+						   cbdata->key,
+						   (GnomeKeyringOperationGetAttributesCallback) pdm_dialog_password_add_item_attrs_cb,
+						   g_memdup (cbdata, sizeof (PdmCallBackData)),
+						   (GDestroyNotify) g_free);
+
+	}
+
+}
+
 static void
 pdm_dialog_password_add (PdmActionInfo *info,
 			 gpointer data)
 {
-	EphyPasswordInfo *pinfo = (EphyPasswordInfo *) data;
-	GtkListStore *store;
-	GtkTreeIter iter;
-	GValue value = { 0, };
+	PdmCallBackData *cbdata;
+	guint key_id = GPOINTER_TO_UINT(data);
 
-	store = GTK_LIST_STORE (info->model);
+	/*
+	 * We have the item id of the password. We will have to check if this
+	 * password entry is of the right type and then can proceed to get the
+	 * the private information. Seahorse is treating every protocol that
+	 * starts with http as Web Access and we will do the same here.
+	 */
 
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store,
-			    &iter,
-			    COL_PASSWORDS_HOST, pinfo->host,
-			    COL_PASSWORDS_USER, pinfo->username,
-			    COL_PASSWORDS_PASSWORD, pinfo->password,
-			    -1);
+	cbdata = g_malloc (sizeof (PdmCallBackData));
+	cbdata->key = key_id;
+	cbdata->store = GTK_LIST_STORE (info->model);
 
-	g_value_init (&value, EPHY_TYPE_PASSWORD_INFO);
-	g_value_take_boxed (&value, pinfo);
-	gtk_list_store_set_value (store, &iter, COL_PASSWORDS_DATA, &value);
-	g_value_unset (&value);
+	/* Get the type of the key_id */
+	gnome_keyring_item_get_info_full (GNOME_KEYRING_DEFAULT,
+					  key_id,
+					  GNOME_KEYRING_ITEM_INFO_BASICS,
+					  (GnomeKeyringOperationGetItemInfoCallback) pdm_dialog_password_add_item_info_cb,
+					  cbdata,
+					  (GDestroyNotify) g_free);
+}
+
+static void
+pdm_dialog_password_remove_dummy_cb (GnomeKeyringResult result,
+				     gpointer data)
+{
 }
 
 static void
@@ -1342,23 +1468,12 @@ pdm_dialog_password_remove (PdmActionInfo *info,
 			    gpointer data)
 {
 	EphyPasswordInfo *pinfo = (EphyPasswordInfo *) data;
-	EphyPasswordManager *manager;
 
-	manager = EPHY_PASSWORD_MANAGER (ephy_embed_shell_get_embed_single
-			(EPHY_EMBED_SHELL (ephy_shell)));
-
-	/* we don't remove the password from the liststore in the callback
-	 * like we do for cookies, since the callback doesn't carry that
-	 * information, and we'd have to reload the whole list, losing the
-	 * selection in the process.
-	 */
-	g_signal_handlers_block_by_func
-		(manager, G_CALLBACK (passwords_changed_cb), info->dialog);
-
-	ephy_password_manager_remove_password (manager, pinfo);
-
-	g_signal_handlers_unblock_by_func
-		(manager, G_CALLBACK (passwords_changed_cb), info->dialog);
+	gnome_keyring_item_delete (GNOME_KEYRING_DEFAULT,
+				   pinfo->keyring_id,
+				   (GnomeKeyringOperationDoneCallback) pdm_dialog_password_remove_dummy_cb,
+				   NULL,
+				   NULL);
 }
 
 /* common routines */
@@ -1375,7 +1490,6 @@ sync_notebook_tab (GtkWidget *notebook,
 	if (page_num == 0 && priv->cookies->filled == FALSE)
 	{
 		priv->cookies->fill (priv->cookies);
-
 		priv->cookies->scroll_to (priv->cookies);
 	}
 	else if (page_num == 1 && priv->passwords->filled == FALSE)
@@ -1398,7 +1512,7 @@ pdm_dialog_response_cb (GtkDialog *widget,
 	{
 		int page;
 		GtkWidget *parent;
-		
+
 		parent = ephy_dialog_get_control (EPHY_DIALOG (dialog),
 						  properties[PROP_WINDOW].id);
 
@@ -1406,14 +1520,14 @@ pdm_dialog_response_cb (GtkDialog *widget,
 		if (page == 0)
 		{
 			/* Cookies */
-			pdm_dialog_show_clear_all_dialog (EPHY_DIALOG (dialog), 
+			pdm_dialog_show_clear_all_dialog (EPHY_DIALOG (dialog),
 							  parent, CLEAR_ALL_COOKIES);
 		}
 		if (page == 1)
 		{
 			/* Passwords */
 			pdm_dialog_show_clear_all_dialog (EPHY_DIALOG (dialog),
-							  parent, CLEAR_ALL_PASSWORDS);			
+							  parent, CLEAR_ALL_PASSWORDS);
 		}
 		return;
 	}
@@ -1432,7 +1546,7 @@ pdm_dialog_init (PdmDialog *dialog)
 
 	ephy_dialog_construct (EPHY_DIALOG(dialog),
 			       properties,
-			       ephy_file ("epiphany.glade"),
+			       ephy_file ("epiphany.ui"),
 			       "pdm_dialog",
 			       NULL);
 
@@ -1447,7 +1561,7 @@ pdm_dialog_init (PdmDialog *dialog)
 
 	g_signal_connect (window, "response",
 			  G_CALLBACK (pdm_dialog_response_cb), dialog);
-	/**
+	/*
 	 * Group all Properties and Remove buttons in the same size group to
 	 * avoid the little jerk you get otherwise when switching pages because
 	 * one set of buttons is wider than another.
@@ -1507,6 +1621,8 @@ pdm_dialog_finalize (GObject *object)
 	g_signal_handlers_disconnect_matched
 		(single, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, object);
 
+	g_signal_handlers_disconnect_by_func (get_cookie_jar (), cookie_changed_cb, object);
+
 	dialog->priv->cookies->destruct (dialog->priv->cookies);
 	dialog->priv->passwords->destruct (dialog->priv->passwords);
 
